package usi.dbdp.portal.util;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import usi.dbdp.portal.util.CommonUtil;

public class Lunar {

   private int year;
   public int month;
   public int day;
   public int weekDay;
   private boolean leap;
   public static final String[] CHINESE_NUMBER = new String[]{"一", "二", "三", "四", "五", "六", "七", "八", "九", "十", "十一", "十二"};
   public final String[] dayNames = new String[]{"星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六"};
   static SimpleDateFormat chineseDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
   static final long[] LUNAR_INFO = new long[]{19416L, 19168L, 42352L, 21717L, 53856L, 55632L, 91476L, 22176L, 39632L, 21970L, 19168L, 42422L, 42192L, 53840L, 119381L, 46400L, 54944L, 44450L, 38320L, 84343L, 18800L, 42160L, 46261L, 27216L, 27968L, 109396L, 11104L, 38256L, 21234L, 18800L, 25958L, 54432L, 59984L, 28309L, 23248L, 11104L, 100067L, 37600L, 116951L, 51536L, 54432L, 120998L, 46416L, 22176L, 107956L, 9680L, 37584L, 53938L, 43344L, 46423L, 27808L, 46416L, 86869L, 19872L, 42448L, 83315L, 21200L, 43432L, 59728L, 27296L, 44710L, 43856L, 19296L, 43748L, 42352L, 21088L, 62051L, 55632L, 23383L, 22176L, 38608L, 19925L, 19152L, 42192L, 54484L, 53840L, 54616L, 46400L, 46496L, 103846L, 38320L, 18864L, 43380L, 42160L, 45690L, 27216L, 27968L, 44870L, 43872L, 38256L, 19189L, 18800L, 25776L, 29859L, 59984L, 27480L, 21952L, 43872L, 38613L, 37600L, 51552L, 55636L, 54432L, 55888L, 30034L, 22176L, 43959L, 9680L, 37584L, 51893L, 43344L, 46240L, 47780L, 44368L, 21977L, 19360L, 42416L, 86390L, 21168L, 43312L, 31060L, 27296L, 44368L, 23378L, 19296L, 42726L, 42208L, 53856L, 60005L, 54576L, 23200L, 30371L, 38608L, 19415L, 19152L, 42192L, 118966L, 53840L, 54560L, 56645L, 46496L, 22224L, 21938L, 18864L, 42359L, 42160L, 43600L, 111189L, 27936L, 44448L};


   private static final int yearDays(int y) {
      int sum = 348;

      for(int i = '耀'; i > 8; i >>= 1) {
         if((LUNAR_INFO[y - 1900] & (long)i) != 0L) {
            ++sum;
         }
      }

      return sum + leapDays(y);
   }

   private static final int leapDays(int y) {
      return leapMonth(y) != 0?((LUNAR_INFO[y - 1900] & 65536L) != 0L?30:29):0;
   }

   private static final int leapMonth(int y) {
      return (int)(LUNAR_INFO[y - 1900] & 15L);
   }

   private static final int monthDays(int y, int m) {
      return (LUNAR_INFO[y - 1900] & (long)(65536 >> m)) == 0L?29:30;
   }

   public final String animalsYear() {
      String[] animals = new String[]{"鼠", "牛", "虎", "兔", "龙", "蛇", "马", "羊", "猴", "鸡", "狗", "猪"};
      return animals[(this.year - 4) % 12];
   }

   private static final String cyclicalm(int num) {
      String[] gan = new String[]{"甲", "乙", "丙", "丁", "戊", "己", "庚", "辛", "壬", "癸"};
      String[] zhi = new String[]{"子", "丑", "寅", "卯", "辰", "巳", "午", "未", "申", "酉", "戌", "亥"};
      return gan[num % 10] + zhi[num % 12];
   }

   public final String cyclical() {
      int num = this.year - 1900 + 36;
      return cyclicalm(num);
   }

   public Lunar(Calendar cal) {
      boolean leapMonth = false;
      Date baseDate = CommonUtil.parse("1900年1月31日", "yyyy年MM月dd日");
      this.weekDay = cal.get(7);
      int offset = (int)((cal.getTime().getTime() - baseDate.getTime()) / 86400000L);
      int daysOfYear = 0;

      int iYear;
      for(iYear = 1900; iYear < 2050 && offset > 0; ++iYear) {
         daysOfYear = yearDays(iYear);
         offset -= daysOfYear;
      }

      if(offset < 0) {
         offset += daysOfYear;
         --iYear;
      }

      this.year = iYear;
      int var9 = leapMonth(iYear);
      this.leap = false;
      int daysOfMonth = 0;

      int iMonth;
      for(iMonth = 1; iMonth < 13 && offset > 0; ++iMonth) {
         if(var9 > 0 && iMonth == var9 + 1 && !this.leap) {
            --iMonth;
            this.leap = true;
            daysOfMonth = leapDays(this.year);
         } else {
            daysOfMonth = monthDays(this.year, iMonth);
         }

         offset -= daysOfMonth;
         if(this.leap && iMonth == var9 + 1) {
            this.leap = false;
         }
      }

      if(offset == 0 && var9 > 0 && iMonth == var9 + 1) {
         if(this.leap) {
            this.leap = false;
         } else {
            this.leap = true;
            --iMonth;
         }
      }

      if(offset < 0) {
         offset += daysOfMonth;
         --iMonth;
      }

      this.month = iMonth;
      this.day = offset + 1;
   }

   public static String getChinaDayString(int day) {
      String[] chineseTen = new String[]{"初", "十", "廿", "卅"};
      int n = day % 10 == 0?9:day % 10 - 1;
      return day > 30?"":(day == 10?"初十":chineseTen[day / 10] + CHINESE_NUMBER[n]);
   }

   public String toString() {
      return "▪公历：" + CommonUtil.format(new Date(), "yyyy年MM月dd日") + " " + this.dayNames[this.weekDay - 1 == -1?0:this.weekDay - 1] + "\n▪农历：" + this.cyclical() + "[" + this.animalsYear() + "年]" + (this.leap?"闰":"") + CHINESE_NUMBER[this.month - 1] + "月" + getChinaDayString(this.day);
   }

   public static void main(String[] args) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(new Date());
      Lunar lunar = new Lunar(calendar);
      System.out.println(lunar.dayNames[lunar.weekDay - 1 == -1?0:lunar.weekDay - 1]);
      System.out.println(CHINESE_NUMBER[lunar.month - 1] + "月" + getChinaDayString(lunar.day));
   }
}
